"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const CircularProgress_1 = tslib_1.__importDefault(require("@material-ui/core/CircularProgress"));
const Paper_1 = tslib_1.__importDefault(require("@material-ui/core/Paper"));
const React = require("react");
const react_redux_1 = require("react-redux");
const services_1 = require("../services");
const store_1 = require("../store");
const page_widgets_1 = require("./page-widgets");
const style_1 = require("./page-widgets/style");
/**
 * maps state fields from the store to component props
 * @param state the redux state
 */
const mapStateToProps = (state, ownProps) => {
    return {
        documentData: state.sensenetDocumentViewer.documentState.document,
        version: state.sensenetDocumentViewer.documentState.version,
        page: state.sensenetDocumentViewer.previewImages.AvailableImages[ownProps.imageIndex - 1] || {},
        activePages: state.sensenetDocumentViewer.viewer.activePages,
        showWatermark: state.sensenetDocumentViewer.viewer.showWatermark,
        pollInterval: state.sensenetDocumentViewer.previewImages.pollInterval,
        isPlacingCommentMarker: state.comments.isPlacingCommentMarker,
    };
};
/**
 * maps state actions from the store to component props
 * @param state the redux state
 */
const mapDispatchToProps = {
    previewAvailable: store_1.previewAvailable,
};
/**
 * Represents a Page component that show the preview image on a Paper component
 * @extends {React.Component<componentType<typeof mapStateToProps, typeof mapDispatchToProps, OwnProps>, PageState>}
 */
class PageComponent extends React.Component {
    constructor(props) {
        super(props);
        this.pollPreview = setInterval(() => {
            if (this.state.isPolling) {
                this.props.previewAvailable(this.props.documentData, this.props.version, this.props.imageIndex);
            }
        }, this.props.pollInterval);
        this.state = {};
    }
    /** event before the component did unmount */
    componentWillUnmount() {
        this.stopPolling();
    }
    stopPolling() {
        if (this.pollPreview) {
            clearInterval(this.pollPreview);
            this.pollPreview = undefined;
        }
    }
    handleMarkerPlacement(event) {
        if (!this.props.isPlacingCommentMarker) {
            return;
        }
        const draftCommentMarker = {
            x: event.nativeEvent.offsetX / this.state.zoomRatio - style_1.MARKER_SIZE,
            y: event.nativeEvent.offsetY / this.state.zoomRatio - style_1.MARKER_SIZE,
            id: 'draft',
        };
        this.setState(Object.assign({}, this.state, { draftCommentMarker }));
        this.props.handleMarkerCreation && this.props.handleMarkerCreation(draftCommentMarker);
    }
    /**
     * Returns a derived state from the specified props
     * @param props The props for state creation
     */
    static getDerivedStateFromProps(props, state) {
        const imageRotation = services_1.ImageUtil.normalizeDegrees((props.page.Attributes && props.page.Attributes.degree) || 0);
        const imageRotationRads = ((imageRotation % 180) * Math.PI) / 180;
        const imgSrc = (props.image === 'preview' ? props.page.PreviewImageUrl : props.page.ThumbnailImageUrl) || '';
        const relativePageSize = services_1.ImageUtil.getImageSize({
            width: props.viewportWidth,
            height: props.viewportHeight,
        }, {
            width: props.page.Width,
            height: props.page.Height,
            rotation: (props.page.Attributes && props.page.Attributes.degree) || 0,
        }, props.zoomMode, props.zoomLevel, props.fitRelativeZoomLevel);
        const boundingBox = services_1.ImageUtil.getRotatedBoundingBoxSize({
            width: props.page.Width,
            height: props.page.Height,
        }, imageRotation);
        const maxDiff = (relativePageSize.height - relativePageSize.width) / 2;
        const diffHeight = Math.sin(imageRotationRads) * maxDiff;
        return {
            draftCommentMarker: props.isPlacingCommentMarker ? state.draftCommentMarker : undefined,
            isActive: props.activePages.indexOf(props.page.Index) >= 0,
            imgSrc,
            pageWidth: relativePageSize.width,
            pageHeight: relativePageSize.height,
            zoomRatio: relativePageSize.height / props.page.Height,
            imageWidth: `${100 * boundingBox.zoomRatio}%`,
            imageHeight: `${100 * boundingBox.zoomRatio}%`,
            imageTransform: `translateY(${diffHeight}px) rotate(${imageRotation}deg)`,
            isPolling: !(imgSrc ? true : false),
        };
    }
    /**
     * renders the component
     */
    render() {
        return (React.createElement(Paper_1.default, { elevation: this.state.isActive ? 8 : 2, className: this.props.elementName, style: { margin: this.props.margin } },
            React.createElement("div", { style: {
                    padding: 0,
                    overflow: 'hidden',
                    width: this.state.pageWidth - 2 * this.props.margin,
                    height: this.state.pageHeight - 2 * this.props.margin,
                    position: 'relative',
                }, onClick: ev => {
                    this.props.onClick(ev);
                    this.handleMarkerPlacement(ev);
                } },
                this.props.showWidgets ? (React.createElement("div", null,
                    React.createElement(page_widgets_1.ShapesWidget, { draftCommentMarker: this.state.draftCommentMarker, zoomRatio: this.state.zoomRatio, page: this.props.page, viewPort: { height: this.state.pageHeight, width: this.state.pageWidth } }))) : null,
                React.createElement("span", { style: { display: 'flex', justifyContent: 'center' } }, this.state.imgSrc ? (React.createElement("img", { src: `${this.state.imgSrc}${this.props.showWatermark ? '?watermark=true' : ''}`, style: {
                        transition: 'transform .1s ease-in-out',
                        width: this.state.imageWidth,
                        height: this.state.imageHeight,
                        transform: this.state.imageTransform,
                    } })) : (React.createElement(CircularProgress_1.default, { style: { marginTop: '50%' } }))))));
    }
}
exports.PageComponent = PageComponent;
const connectedComponent = react_redux_1.connect(mapStateToProps, mapDispatchToProps)(PageComponent);
exports.Page = connectedComponent;
//# sourceMappingURL=Page.js.map