"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const IconButton_1 = tslib_1.__importDefault(require("@material-ui/core/IconButton"));
const Delete_1 = tslib_1.__importDefault(require("@material-ui/icons/Delete"));
const react_1 = tslib_1.__importDefault(require("react"));
const react_redux_1 = require("react-redux");
const store_1 = require("../../store");
/**
 * maps state fields from the store to component props
 * @param state the redux state
 */
const mapStateToProps = (state) => {
    return {
        zoomMode: state.sensenetDocumentViewer.viewer.zoomMode,
        zoomLevel: state.sensenetDocumentViewer.viewer.customZoomLevel,
    };
};
/**
 * maps state actions from the store to component props
 * @param state the redux state
 */
const mapDispatchToProps = {
    updateShapeData: store_1.updateShapeData,
    removeShape: store_1.removeShape,
};
/**
 * Class for displaying Shapes on a document page
 */
class ShapeComponent extends react_1.default.Component {
    constructor() {
        super(...arguments);
        /** the component state */
        this.state = {
            focused: false,
            onResized: this.onResized.bind(this),
            onBlur: this.onBlur.bind(this),
            onDragStart: this.onDragStart.bind(this),
            handleKeyPress: this.handleKeyPress.bind(this),
            onFocus: this.onFocus.bind(this),
        };
    }
    /**
     * Method that returns the shape dimensions as CSS properties
     * @param shape the shape instance
     * @param offsetX optional X offset
     * @param offsetY optional Y offset
     */
    getShapeDimensions(shape, offsetX = 0, offsetY = 0) {
        return {
            top: shape.y * this.props.zoomRatio + offsetY * this.props.zoomRatio,
            left: shape.x * this.props.zoomRatio + offsetX * this.props.zoomRatio,
            width: shape.w * this.props.zoomRatio,
            height: shape.h * this.props.zoomRatio,
        };
    }
    /** event that will be triggered on resize */
    onResized(ev) {
        const boundingBox = ev.currentTarget.getBoundingClientRect();
        const [shape, shapeType, zoomRatio] = [this.props.shape, this.props.shapeType, this.props.zoomRatio];
        const newSize = {
            w: boundingBox.width * (1 / zoomRatio),
            h: boundingBox.height * (1 / zoomRatio),
        };
        if (Math.abs(newSize.w - shape.w) > 1 || Math.abs(newSize.h - shape.h) > 1) {
            this.props.updateShapeData(shapeType, shape.guid, Object.assign({}, shape, newSize));
        }
    }
    /** onDragStart event handler for the Shape instance */
    onDragStart(ev) {
        const additionalOffset = this.props.additionalOffset || { width: 0, height: 0 };
        ev.dataTransfer.setData('shape', JSON.stringify({
            type: this.shapeType,
            shape: this.props.shape,
            additionalOffset,
            offset: {
                width: ev.clientX - ev.currentTarget.getBoundingClientRect().left + additionalOffset.width,
                height: ev.clientY - ev.currentTarget.getBoundingClientRect().top + additionalOffset.height,
            },
        }));
    }
    /** onKeyPress event handler for deleting shapes */
    handleKeyPress(ev) {
        switch (ev.key) {
            case 'Backspace':
            case 'Delete':
                this.props.canEdit && this.props.removeShape(this.props.shapeType, this.props.shape.guid);
                break;
            // no default
        }
    }
    /** onFocus event handler that updates the 'focused' property */
    onFocus() {
        if (!this.state.focused) {
            this.setState(Object.assign({}, this.state, { focused: true }));
        }
    }
    /** onBlur event handler that updates the 'focused' property */
    onBlur(ev) {
        if (!ev.currentTarget.contains(ev.nativeEvent.relatedTarget)) {
            this.setState(Object.assign({}, this.state, { focused: false }));
        }
    }
    /**
     * renders the component
     */
    render() {
        return (react_1.default.createElement("div", { onClickCapture: ev => ev.stopPropagation(), style: { filter: this.state.focused ? 'contrast(.9) brightness(1.1)' : '' }, onKeyUp: this.state.handleKeyPress, onFocus: this.state.onFocus, onBlur: this.state.onBlur }, this.renderShape()));
    }
}
class ShapeRedaction extends ShapeComponent {
    constructor() {
        super(...arguments);
        /** type of the Shape */
        this.shapeType = 'redactions';
    }
    /** renders the Shape component */
    renderShape() {
        {
            return (react_1.default.createElement("div", { tabIndex: 0, draggable: this.props.canEdit, onDragStart: this.state.onDragStart, key: `r-${this.props.shape.h}-${this.props.shape.w}`, onMouseUp: this.state.onResized, style: Object.assign({}, this.getShapeDimensions(this.props.shape), { resize: this.props.canEdit ? 'both' : 'none', position: 'absolute', overflow: 'auto', backgroundColor: 'black' }) }));
        }
    }
}
const shapeRedaction = react_redux_1.connect(mapStateToProps, mapDispatchToProps)(ShapeRedaction);
exports.ShapeRedaction = shapeRedaction;
class ShapeAnnotation extends ShapeComponent {
    constructor() {
        super(...arguments);
        /** type of the Shape */
        this.shapeType = 'annotations';
    }
    /** keypress handler for to stop event bubbling and shape deletion */
    handleKeyPress(_ev) {
        /** */
    }
    /** onBlur handler for updating inner text */
    onBlur(ev) {
        super.onBlur(ev);
        this.props.updateShapeData(this.shapeType, this.props.shape.guid, Object.assign({}, this.props.shape, { text: ev.currentTarget.innerText.trim() }));
    }
    /** renders the Shape component */
    renderShape() {
        {
            return (react_1.default.createElement("div", null,
                react_1.default.createElement("div", { onKeyUp: ev => this.handleKeyPress(ev), tabIndex: 0, draggable: this.props.canEdit, onDragStart: this.state.onDragStart, onMouseUp: this.state.onResized, style: Object.assign({}, this.getShapeDimensions(this.props.shape, -120, 0), { position: 'absolute', resize: this.props.canEdit ? 'both' : 'none', overflow: 'hidden', backgroundColor: 'blanchedalmond', lineHeight: `${this.props.shape.lineHeight * this.props.zoomRatio}pt`, fontWeight: this.props.shape.fontBold, color: this.props.shape.fontColor, fontFamily: this.props.shape.fontFamily, fontSize: parseFloat(this.props.shape.fontSize.replace('pt', '')) * this.props.zoomRatio, fontStyle: this.props.shape.fontItalic, boxShadow: `${5 * this.props.zoomRatio}px ${5 * this.props.zoomRatio}px ${15 *
                            this.props.zoomRatio}px rgba(0,0,0,.3)`, padding: `${10 * this.props.zoomRatio}pt`, boxSizing: 'border-box' }) },
                    react_1.default.createElement("div", { style: { width: '100%', height: '100%', overflow: 'auto' }, contentEditable: this.props.canEdit ? 'plaintext-only' : false, suppressContentEditableWarning: true }, this.props.shape.text),
                    this.state.focused ? (react_1.default.createElement("div", { style: { position: 'relative', top: `-${64 * this.props.zoomRatio}px` } },
                        react_1.default.createElement(IconButton_1.default, null,
                            react_1.default.createElement(Delete_1.default, { style: { color: 'black' }, scale: this.props.zoomRatio, onMouseUp: () => this.props.removeShape(this.shapeType, this.props.shape.guid) })))) : null)));
        }
    }
}
const shapeAnnotation = react_redux_1.connect(mapStateToProps, mapDispatchToProps)(ShapeAnnotation);
exports.ShapeAnnotation = shapeAnnotation;
class ShapeHighlight extends ShapeComponent {
    constructor() {
        super(...arguments);
        /** type of the Shape */
        this.shapeType = 'highlights';
    }
    /** renders the Highlight shape component */
    renderShape() {
        {
            return (react_1.default.createElement("div", { tabIndex: 0, draggable: this.props.canEdit, onDragStart: this.state.onDragStart, onMouseUp: this.state.onResized, style: Object.assign({}, this.getShapeDimensions(this.props.shape), { position: 'absolute', resize: this.props.canEdit ? 'both' : 'none', overflow: 'auto', backgroundColor: 'yellow', opacity: 0.5 }) }));
        }
    }
}
const shapeHighlight = react_redux_1.connect(mapStateToProps, mapDispatchToProps)(ShapeHighlight);
exports.ShapeHighlight = shapeHighlight;
//# sourceMappingURL=Shape.js.map