import { Reducer } from 'redux';
import { IInjectableActionCallbackParams } from 'redux-di-middleware';
import { DocumentData, Shape, Shapes } from '../models';
import { Dimensions } from '../services';
import { RootReducerType } from './RootReducer';
/**
 * Model for the document state
 */
export interface DocumentStateType {
    pollInterval: number;
    idOrPath?: number | string;
    version?: string;
    document: DocumentData;
    error?: string;
    isLoading: boolean;
    canEdit: boolean;
    canHideWatermark: boolean;
    canHideRedaction: boolean;
    hasChanges: boolean;
}
/**
 * Resets the document viewer state to the default.
 */
export declare const resetDocumentData: () => {
    type: string;
};
/**
 * Action that updates the store with the received document data
 * @param document The received document data
 */
export declare const documentReceivedAction: (document: DocumentData) => {
    type: string;
    document: DocumentData;
};
/**
 * Action that updates the store with a document receive error
 * @param error The error message
 */
export declare const documentReceiveErrorAction: (error: any) => {
    type: string;
    error: any;
};
/**
 * Action that updates the store with the given permission values
 * @param canEdit 'Can edit' permission value
 * @param canHideRedaction 'Can hide redaction' permission value
 * @param canHideWatermark 'Can hide watermark' permission value
 */
export declare const documentPermissionsReceived: (canEdit: boolean, canHideRedaction: boolean, canHideWatermark: boolean) => {
    type: string;
    canEdit: boolean;
    canHideRedaction: boolean;
    canHideWatermark: boolean;
};
/**
 * Thunk action that polls document data from the specified API
 * @param hostName the host name, e.g. 'https://my-sensenet-site.com'
 * @param idOrPath Id or full path for the document, e.g.: 'Root/Sites/MySite/MyDocLib/('doc.docx')
 * @param version The document version
 */
export declare const pollDocumentData: (hostName: string, idOrPath: string | number, version?: string) => {
    type: string;
    inject: (options: IInjectableActionCallbackParams<RootReducerType>) => Promise<void>;
};
/**
 * Action that updates a provided shape data
 * @param shapeType the type of the shape
 * @param shapeGuid the unique identifyer for the shape
 * @param shapeData the new shape data
 */
export declare const updateShapeData: <K extends "redactions" | "highlights" | "annotations">(shapeType: K, shapeGuid: string, shapeData: Shapes[K][0]) => {
    type: string;
    shapeType: K;
    shapeGuid: string;
    shapeData: Shapes[K][0];
};
/**
 * Action that removes a specified shape
 * @param shapeType The type of the shape
 * @param shapeGuid The unique identifier for the shape
 */
export declare const removeShape: <K extends "redactions" | "highlights" | "annotations">(shapeType: K, shapeGuid: string) => {
    type: string;
    shapeType: K;
    shapeGuid: string;
};
/**
 * Action that updates the polling interval
 * @param pollInterval The interval in millisecs
 */
export declare const setPollInterval: (pollInterval: number) => {
    type: string;
    pollInterval: number;
};
/**
 * Action that will be fired when a save request has been sent
 */
export declare const saveChangesRequest: () => {
    type: string;
};
/**
 * Action that will be fired on save error
 * @param error The error value
 */
export declare const saveChangesError: (error: any) => {
    type: string;
    error: any;
};
/**
 * Action that will be fired when saving succeeded
 */
export declare const saveChangesSuccess: () => {
    type: string;
};
/**
 * Action that rotates the specified shapes for the given page(s)
 * @param pages The pages to rotate
 * @param degree The rotation angle in degrees
 */
export declare const rotateShapesForPages: (pages: {
    index: number;
    size: Dimensions;
}[], degree: number) => {
    type: string;
    pages: {
        index: number;
        size: Dimensions;
    }[];
    degree: number;
};
/**
 * Thunk action to call the Save endpoint with the current document state to save changes
 */
export declare const saveChanges: () => {
    type: string;
    inject: (options: IInjectableActionCallbackParams<RootReducerType>) => Promise<void>;
};
/**
 * helper method to apply shape rotations
 * @param shapes the shape(s) to rotate
 * @param degree the rotation angle in degrees
 * @param pages the page info
 */
export declare const applyShapeRotations: <T extends Shape>(shapes: T[], degree: number, pages: {
    index: number;
    size: Dimensions;
}[]) => T[];
/**
 * The default state data for the Document
 */
export declare const defaultState: DocumentStateType;
/**
 * Reducer method for the document state
 * @param state the current state
 * @param action the action to dispatch
 */
export declare const documentStateReducer: Reducer<DocumentStateType>;
//# sourceMappingURL=Document.d.ts.map